# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: permutive.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fpermutive.proto\x12\tpermutive\"\x0c\n\nImportRole\"1\n\nExportRole\x12#\n\x05input\x18\x01 \x01(\x0b\x32\x14.permutive.SinkInput\"\x1a\n\nSingleFile\x12\x0c\n\x04name\x18\x01 \x01(\t\"\t\n\x07RawFile\"C\n\x07ZipFile\x12+\n\nsingleFile\x18\x01 \x01(\x0b\x32\x15.permutive.SingleFileH\x00\x42\x0b\n\tselection\"{\n\tSinkInput\x12\x12\n\ndependency\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12!\n\x03raw\x18\x03 \x01(\x0b\x32\x12.permutive.RawFileH\x00\x12!\n\x03zip\x18\x04 \x01(\x0b\x32\x12.permutive.ZipFileH\x00\x42\x06\n\x04\x66ile\"\xf4\x01\n\x1cPermutiveWorkerConfiguration\x12\x1d\n\x15\x63redentialsDependency\x18\x01 \x01(\t\x12+\n\nimportRole\x18\x02 \x01(\x0b\x32\x15.permutive.ImportRoleH\x00\x12+\n\nexportRole\x18\x03 \x01(\x0b\x32\x15.permutive.ExportRoleH\x00\x12\x10\n\x08importId\x18\x04 \x01(\t\x12\x13\n\x0bsegmentName\x18\x05 \x01(\t\x12\x13\n\x0bsegmentCode\x18\x06 \x01(\t\x12\x17\n\x0finputHasHeaders\x18\x07 \x01(\x08\x42\x06\n\x04roleb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'permutive_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _IMPORTROLE._serialized_start=30
  _IMPORTROLE._serialized_end=42
  _EXPORTROLE._serialized_start=44
  _EXPORTROLE._serialized_end=93
  _SINGLEFILE._serialized_start=95
  _SINGLEFILE._serialized_end=121
  _RAWFILE._serialized_start=123
  _RAWFILE._serialized_end=132
  _ZIPFILE._serialized_start=134
  _ZIPFILE._serialized_end=201
  _SINKINPUT._serialized_start=203
  _SINKINPUT._serialized_end=326
  _PERMUTIVEWORKERCONFIGURATION._serialized_start=329
  _PERMUTIVEWORKERCONFIGURATION._serialized_end=573
# @@protoc_insertion_point(module_scope)
