# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: data_room.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import attestation_pb2 as attestation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0f\x64\x61ta_room.proto\x12\tdata_room\x1a\x11\x61ttestation.proto\"\xb4\x01\n\x08\x44\x61taRoom\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x39\n\x12governanceProtocol\x18\x04 \x01(\x0b\x32\x1d.data_room.GovernanceProtocol\x12>\n\x14initialConfiguration\x18\x05 \x01(\x0b\x32 .data_room.DataRoomConfiguration\"\xb6\x01\n\x12GovernanceProtocol\x12?\n\x14staticDataRoomPolicy\x18\x01 \x01(\x0b\x32\x1f.data_room.StaticDataRoomPolicyH\x00\x12U\n\x1f\x61\x66\x66\x65\x63tedDataOwnersApprovePolicy\x18\x02 \x01(\x0b\x32*.data_room.AffectedDataOwnersApprovePolicyH\x00\x42\x08\n\x06policy\"\x16\n\x14StaticDataRoomPolicy\"!\n\x1f\x41\x66\x66\x65\x63tedDataOwnersApprovePolicy\"J\n\x15\x44\x61taRoomConfiguration\x12\x31\n\x08\x65lements\x18\x01 \x03(\x0b\x32\x1f.data_room.ConfigurationElement\"\x9d\x02\n\x14\x43onfigurationElement\x12\n\n\x02id\x18\x01 \x01(\t\x12-\n\x0b\x63omputeNode\x18\x02 \x01(\x0b\x32\x16.data_room.ComputeNodeH\x00\x12I\n\x18\x61ttestationSpecification\x18\x03 \x01(\x0b\x32%.attestation.AttestationSpecificationH\x00\x12\x33\n\x0euserPermission\x18\x04 \x01(\x0b\x32\x19.data_room.UserPermissionH\x00\x12?\n\x14\x61uthenticationMethod\x18\x05 \x01(\x0b\x32\x1f.data_room.AuthenticationMethodH\x00\x42\t\n\x07\x65lement\"\xb8\x01\n\x19\x43onfigurationModification\x12)\n\x03\x61\x64\x64\x18\x01 \x01(\x0b\x32\x1a.data_room.AddModificationH\x00\x12/\n\x06\x63hange\x18\x02 \x01(\x0b\x32\x1d.data_room.ChangeModificationH\x00\x12/\n\x06\x64\x65lete\x18\x03 \x01(\x0b\x32\x1d.data_room.DeleteModificationH\x00\x42\x0e\n\x0cmodification\"C\n\x0f\x41\x64\x64Modification\x12\x30\n\x07\x65lement\x18\x01 \x01(\x0b\x32\x1f.data_room.ConfigurationElement\"F\n\x12\x43hangeModification\x12\x30\n\x07\x65lement\x18\x01 \x01(\x0b\x32\x1f.data_room.ConfigurationElement\" \n\x12\x44\x65leteModification\x12\n\n\x02id\x18\x01 \x01(\t\"\x9c\x01\n\x13\x43onfigurationCommit\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\ndataRoomId\x18\x03 \x01(\x0c\x12\x1a\n\x12\x64\x61taRoomHistoryPin\x18\x04 \x01(\x0c\x12;\n\rmodifications\x18\x05 \x03(\x0b\x32$.data_room.ConfigurationModification\"O\n\x18WindowRateLimitingConfig\x12\x19\n\x11timeWindowSeconds\x18\x01 \x01(\r\x12\x18\n\x10numMaxExecutions\x18\x02 \x01(\r\"U\n\x12RateLimitingConfig\x12\x35\n\x06window\x18\x01 \x01(\x0b\x32#.data_room.WindowRateLimitingConfigH\x00\x42\x08\n\x06method\"\xb6\x02\n\x0b\x43omputeNode\x12\x10\n\x08nodeName\x18\x01 \x01(\t\x12*\n\x04leaf\x18\x02 \x01(\x0b\x32\x1a.data_room.ComputeNodeLeafH\x00\x12\x34\n\tparameter\x18\x04 \x01(\x0b\x32\x1f.data_room.ComputeNodeParameterH\x00\x12.\n\x06\x62ranch\x18\x03 \x01(\x0b\x32\x1c.data_room.ComputeNodeBranchH\x00\x12\x30\n\x07\x61irlock\x18\x06 \x01(\x0b\x32\x1d.data_room.ComputeNodeAirlockH\x00\x12\x38\n\x0crateLimiting\x18\x05 \x01(\x0b\x32\x1d.data_room.RateLimitingConfigH\x01\x88\x01\x01\x42\x06\n\x04nodeB\x0f\n\r_rateLimiting\"%\n\x0f\x43omputeNodeLeaf\x12\x12\n\nisRequired\x18\x01 \x01(\x08\"*\n\x14\x43omputeNodeParameter\x12\x12\n\nisRequired\x18\x01 \x01(\x08\"E\n\x12\x43omputeNodeAirlock\x12\x12\n\nquotaBytes\x18\x01 \x01(\x04\x12\x1b\n\x13\x61irlockedDependency\x18\x02 \x01(\t\"&\n\x13\x43omputeNodeProtocol\x12\x0f\n\x07version\x18\x01 \x01(\r\"\xc3\x01\n\x11\x43omputeNodeBranch\x12\x0e\n\x06\x63onfig\x18\x01 \x01(\x0c\x12\x14\n\x0c\x64\x65pendencies\x18\x02 \x03(\t\x12\x32\n\x0coutputFormat\x18\x03 \x01(\x0e\x32\x1c.data_room.ComputeNodeFormat\x12\x30\n\x08protocol\x18\x04 \x01(\x0b\x32\x1e.data_room.ComputeNodeProtocol\x12\"\n\x1a\x61ttestationSpecificationId\x18\x05 \x01(\t\"k\n\x0eUserPermission\x12\r\n\x05\x65mail\x18\x01 \x01(\t\x12*\n\x0bpermissions\x18\x02 \x03(\x0b\x32\x15.data_room.Permission\x12\x1e\n\x16\x61uthenticationMethodId\x18\x03 \x01(\t\"\x95\x01\n\x14\x41uthenticationMethod\x12)\n\x0bpersonalPki\x18\x01 \x01(\x0b\x32\x14.data_room.PkiPolicy\x12#\n\x05\x64qPki\x18\x02 \x01(\x0b\x32\x14.data_room.PkiPolicy\x12-\n\tdcrSecret\x18\x03 \x01(\x0b\x32\x1a.data_room.DcrSecretPolicy\"\'\n\tPkiPolicy\x12\x1a\n\x12rootCertificatePem\x18\x01 \x01(\x0c\"&\n\x0f\x44\x63rSecretPolicy\x12\x13\n\x0b\x64\x63rSecretId\x18\x01 \x01(\x0c\"\x8c\t\n\nPermission\x12G\n\x18\x65xecuteComputePermission\x18\x01 \x01(\x0b\x32#.data_room.ExecuteComputePermissionH\x00\x12;\n\x12leafCrudPermission\x18\x02 \x01(\x0b\x32\x1d.data_room.LeafCrudPermissionH\x00\x12K\n\x1aretrieveDataRoomPermission\x18\x03 \x01(\x0b\x32%.data_room.RetrieveDataRoomPermissionH\x00\x12K\n\x1aretrieveAuditLogPermission\x18\x04 \x01(\x0b\x32%.data_room.RetrieveAuditLogPermissionH\x00\x12W\n retrieveDataRoomStatusPermission\x18\x05 \x01(\x0b\x32+.data_room.RetrieveDataRoomStatusPermissionH\x00\x12S\n\x1eupdateDataRoomStatusPermission\x18\x06 \x01(\x0b\x32).data_room.UpdateDataRoomStatusPermissionH\x00\x12]\n#retrievePublishedDatasetsPermission\x18\x07 \x01(\x0b\x32..data_room.RetrievePublishedDatasetsPermissionH\x00\x12\x37\n\x10\x64ryRunPermission\x18\x08 \x01(\x0b\x32\x1b.data_room.DryRunPermissionH\x00\x12W\n generateMergeSignaturePermission\x18\t \x01(\x0b\x32+.data_room.GenerateMergeSignaturePermissionH\x00\x12]\n#executeDevelopmentComputePermission\x18\n \x01(\x0b\x32..data_room.ExecuteDevelopmentComputePermissionH\x00\x12[\n\"mergeConfigurationCommitPermission\x18\x0b \x01(\x0b\x32-.data_room.MergeConfigurationCommitPermissionH\x00\x12U\n\x1fretrieveComputeResultPermission\x18\x0c \x01(\x0b\x32*.data_room.RetrieveComputeResultPermissionH\x00\x12M\n\x1b\x63\x61sAuxiliaryStatePermission\x18\r \x01(\x0b\x32&.data_room.CasAuxiliaryStatePermissionH\x00\x12O\n\x1creadAuxiliaryStatePermission\x18\x0e \x01(\x0b\x32\'.data_room.ReadAuxiliaryStatePermissionH\x00\x42\x0c\n\npermission\"1\n\x18\x45xecuteComputePermission\x12\x15\n\rcomputeNodeId\x18\x01 \x01(\t\"(\n\x12LeafCrudPermission\x12\x12\n\nleafNodeId\x18\x01 \x01(\t\"\x1c\n\x1aRetrieveDataRoomPermission\"\x1c\n\x1aRetrieveAuditLogPermission\"\"\n RetrieveDataRoomStatusPermission\" \n\x1eUpdateDataRoomStatusPermission\"%\n#RetrievePublishedDatasetsPermission\"\x12\n\x10\x44ryRunPermission\"\"\n GenerateMergeSignaturePermission\"%\n#ExecuteDevelopmentComputePermission\"$\n\"MergeConfigurationCommitPermission\"8\n\x1fRetrieveComputeResultPermission\x12\x15\n\rcomputeNodeId\x18\x01 \x01(\t\"\x1d\n\x1b\x43\x61sAuxiliaryStatePermission\"\x1e\n\x1cReadAuxiliaryStatePermission*%\n\x11\x43omputeNodeFormat\x12\x07\n\x03RAW\x10\x00\x12\x07\n\x03ZIP\x10\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'data_room_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _COMPUTENODEFORMAT._serialized_start=4259
  _COMPUTENODEFORMAT._serialized_end=4296
  _DATAROOM._serialized_start=50
  _DATAROOM._serialized_end=230
  _GOVERNANCEPROTOCOL._serialized_start=233
  _GOVERNANCEPROTOCOL._serialized_end=415
  _STATICDATAROOMPOLICY._serialized_start=417
  _STATICDATAROOMPOLICY._serialized_end=439
  _AFFECTEDDATAOWNERSAPPROVEPOLICY._serialized_start=441
  _AFFECTEDDATAOWNERSAPPROVEPOLICY._serialized_end=474
  _DATAROOMCONFIGURATION._serialized_start=476
  _DATAROOMCONFIGURATION._serialized_end=550
  _CONFIGURATIONELEMENT._serialized_start=553
  _CONFIGURATIONELEMENT._serialized_end=838
  _CONFIGURATIONMODIFICATION._serialized_start=841
  _CONFIGURATIONMODIFICATION._serialized_end=1025
  _ADDMODIFICATION._serialized_start=1027
  _ADDMODIFICATION._serialized_end=1094
  _CHANGEMODIFICATION._serialized_start=1096
  _CHANGEMODIFICATION._serialized_end=1166
  _DELETEMODIFICATION._serialized_start=1168
  _DELETEMODIFICATION._serialized_end=1200
  _CONFIGURATIONCOMMIT._serialized_start=1203
  _CONFIGURATIONCOMMIT._serialized_end=1359
  _WINDOWRATELIMITINGCONFIG._serialized_start=1361
  _WINDOWRATELIMITINGCONFIG._serialized_end=1440
  _RATELIMITINGCONFIG._serialized_start=1442
  _RATELIMITINGCONFIG._serialized_end=1527
  _COMPUTENODE._serialized_start=1530
  _COMPUTENODE._serialized_end=1840
  _COMPUTENODELEAF._serialized_start=1842
  _COMPUTENODELEAF._serialized_end=1879
  _COMPUTENODEPARAMETER._serialized_start=1881
  _COMPUTENODEPARAMETER._serialized_end=1923
  _COMPUTENODEAIRLOCK._serialized_start=1925
  _COMPUTENODEAIRLOCK._serialized_end=1994
  _COMPUTENODEPROTOCOL._serialized_start=1996
  _COMPUTENODEPROTOCOL._serialized_end=2034
  _COMPUTENODEBRANCH._serialized_start=2037
  _COMPUTENODEBRANCH._serialized_end=2232
  _USERPERMISSION._serialized_start=2234
  _USERPERMISSION._serialized_end=2341
  _AUTHENTICATIONMETHOD._serialized_start=2344
  _AUTHENTICATIONMETHOD._serialized_end=2493
  _PKIPOLICY._serialized_start=2495
  _PKIPOLICY._serialized_end=2534
  _DCRSECRETPOLICY._serialized_start=2536
  _DCRSECRETPOLICY._serialized_end=2574
  _PERMISSION._serialized_start=2577
  _PERMISSION._serialized_end=3741
  _EXECUTECOMPUTEPERMISSION._serialized_start=3743
  _EXECUTECOMPUTEPERMISSION._serialized_end=3792
  _LEAFCRUDPERMISSION._serialized_start=3794
  _LEAFCRUDPERMISSION._serialized_end=3834
  _RETRIEVEDATAROOMPERMISSION._serialized_start=3836
  _RETRIEVEDATAROOMPERMISSION._serialized_end=3864
  _RETRIEVEAUDITLOGPERMISSION._serialized_start=3866
  _RETRIEVEAUDITLOGPERMISSION._serialized_end=3894
  _RETRIEVEDATAROOMSTATUSPERMISSION._serialized_start=3896
  _RETRIEVEDATAROOMSTATUSPERMISSION._serialized_end=3930
  _UPDATEDATAROOMSTATUSPERMISSION._serialized_start=3932
  _UPDATEDATAROOMSTATUSPERMISSION._serialized_end=3964
  _RETRIEVEPUBLISHEDDATASETSPERMISSION._serialized_start=3966
  _RETRIEVEPUBLISHEDDATASETSPERMISSION._serialized_end=4003
  _DRYRUNPERMISSION._serialized_start=4005
  _DRYRUNPERMISSION._serialized_end=4023
  _GENERATEMERGESIGNATUREPERMISSION._serialized_start=4025
  _GENERATEMERGESIGNATUREPERMISSION._serialized_end=4059
  _EXECUTEDEVELOPMENTCOMPUTEPERMISSION._serialized_start=4061
  _EXECUTEDEVELOPMENTCOMPUTEPERMISSION._serialized_end=4098
  _MERGECONFIGURATIONCOMMITPERMISSION._serialized_start=4100
  _MERGECONFIGURATIONCOMMITPERMISSION._serialized_end=4136
  _RETRIEVECOMPUTERESULTPERMISSION._serialized_start=4138
  _RETRIEVECOMPUTERESULTPERMISSION._serialized_end=4194
  _CASAUXILIARYSTATEPERMISSION._serialized_start=4196
  _CASAUXILIARYSTATEPERMISSION._serialized_end=4225
  _READAUXILIARYSTATEPERMISSION._serialized_start=4227
  _READAUXILIARYSTATEPERMISSION._serialized_end=4257
# @@protoc_insertion_point(module_scope)
